# -*- encoding: utf-8 -*-
"""Host representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib


AWS_AUTO_PROVIDER = 'ec2-auto'
AWS_ON_DEMAND_PROVIDER = 'ec2-ondemand'
DOCKER_PROVIDER = 'docker'
STATIC_PROVIDER = 'static'


class MountPoint(_BaseEvergreenObject):
    """
    Representation of Mount Point in distro settings.
    """
    device_name = evg_attrib('device_name')
    size = evg_attrib('size')
    virtual_name = evg_attrib('virtual_name')

    def __init__(self, json, api):
        """
        Create an instance of the mount point.

        :param json: Json of mount point.
        :param api: Evergreen API.
        """
        super(MountPoint, self).__init__(json, api)


class StaticDistroSettings(_BaseEvergreenObject):
    """
    Representation of Evergreen static distro settings.
    """
    def __init__(self, json, api):
        """
        Create an instance of the distro settings for static images.

        :param json: Json of distro settings.
        :param api: Evergreen API.
        """
        super(StaticDistroSettings, self).__init__(json, api)

    @property
    def hosts(self):
        """
        Retrieves Hosts of static distro.

        :return: hosts of static distro.
        """
        if 'hosts' in self.json:
            return [host['name'] for host in self.json['hosts']]
        return []


class DockerDistroSettings(_BaseEvergreenObject):
    """
    Representation of docker distro settings.
    """
    image_url = evg_attrib('image_url')

    def __init__(self, json, api):
        """
        Create an instance of the distro settings for docker.

        :param json: Json of distro settings.
        :param api: Evergreen API.
        """
        super(DockerDistroSettings, self).__init__(json, api)


class AwsDistroSettings(_BaseEvergreenObject):
    """
    Representation of AWS Distro Settings.
    """
    ami = evg_attrib('ami')
    aws_access_key_id = evg_attrib('aws_access_key_id')
    aws_access_secret_id = evg_attrib('aws_access_secret_id')
    bid_price = evg_attrib('bid_price')
    instance_type = evg_attrib('instance_type')
    ipv6 = evg_attrib('ipv6')
    is_vpc = evg_attrib('is_vpc')
    key_name = evg_attrib('key_name')
    region = evg_attrib('region')
    security_group = evg_attrib('security_group')
    security_group_ids = evg_attrib('security_group_ids')
    subnet_id = evg_attrib('subnet_id')
    user_data = evg_attrib('user_data')
    vpc_name = evg_attrib('vpc_name')

    def __init__(self, json, api):
        """
        Create an instance of the distro settings.

        :param json: Json of the distro settings.
        :param api: Evergreen API.
        """
        super(AwsDistroSettings, self).__init__(json, api)

    @property
    def mount_points(self):
        """
        Retrieve list of mount points for the distro.

        :return: List of mount points.
        """
        if 'mount_points' in self.json:
            return [MountPoint(mp, self._api) for mp in self.json['mount_points']]
        return []


class PlannerSettings(_BaseEvergreenObject):
    """
    Representation of planner settings.
    """
    version = evg_attrib('version')
    minimum_hosts = evg_attrib('minimum_hosts')
    maximum_hosts = evg_attrib('maximum_hosts')
    target_time = evg_attrib('target_time')
    acceptable_host_idle_time = evg_attrib('acceptable_host_idle_time')
    group_versions = evg_attrib('group_versions')
    patch_zipper_factor = evg_attrib('patch_zipper_factor')
    task_ordering = evg_attrib('task_ordering')

    def __init__(self, json, api):
        """
        Create an instance of planner settings for a distro.

        :param json: planner settings json.
        :param api: Evergreen API.
        """
        super(PlannerSettings, self).__init__(json, api)


class FinderSettings(_BaseEvergreenObject):
    """
    Representation of finder settings.
    """
    version = evg_attrib('version')

    def __init__(self, json, api):
        """
        Create an instance of finder settings for a distro.

        :param json: finder settings json.
        :param api: Evergreen API.
        """
        super(FinderSettings, self).__init__(json, api)


class Distro(_BaseEvergreenObject):
    """
    Representation of an Evergreen Distro.
    """
    _PROVIDER_MAP = {
        AWS_ON_DEMAND_PROVIDER: AwsDistroSettings,
        AWS_AUTO_PROVIDER: AwsDistroSettings,
        DOCKER_PROVIDER: DockerDistroSettings,
        STATIC_PROVIDER: StaticDistroSettings,
    }

    name = evg_attrib('name')
    user_spawn_allowed = evg_attrib('user_spawn_allowed')
    provider = evg_attrib('provider')
    image_id = evg_attrib('image_id')
    arch = evg_attrib('arch')
    work_dir = evg_attrib('work_dir')
    pool_size = evg_attrib('pool_size')
    setup_as_sudo = evg_attrib('setup_as_sudo')
    setup = evg_attrib('setup')
    teardown = evg_attrib('teardown')
    user = evg_attrib('user')
    bootstrap_method = evg_attrib('bootstrap_method')
    communication_method = evg_attrib('communication_method')
    clone_method = evg_attrib('clone_method')
    shell_path = evg_attrib('shell_path')
    curator_dir = evg_attrib('curator_dir')
    client_dir = evg_attrib('client_dir')
    jasper_credentials_path = evg_attrib('jasper_credentials_path')
    ssh_key = evg_attrib('ssh_key')
    ssh_options = evg_attrib('ssh_options')
    disabled = evg_attrib('disabled')
    container_pool = evg_attrib('container_pool')

    def __init__(self, json, api):
        """
        Create an instance of a distro.

        :param json: Json of a distro.
        :param api: Evergreen API.
        """
        super(Distro, self).__init__(json, api)
        self._expansions_dict = None

    @property
    def settings(self):
        """
        Retrieve the settings for the distro.

        :return: settings for distro.
        """
        if 'settings' in self.json:
            if self.provider in self._PROVIDER_MAP:
                return self._PROVIDER_MAP[self.provider](self.json['settings'], self._api)
            return self.json['settings']
        return None

    @property
    def expansions(self):
        """
        Retrieve dict of expansions for distro.

        :return: dict of expansions.
        """
        if not self._expansions_dict and 'expansions' in self.json:
            self._expansions_dict = {exp['key']: exp['value'] for exp in self.json['expansions']}
        return self._expansions_dict

    @property
    def planner_settings(self):
        """
        Retrieve planner settings for distro.

        :return: planner settings.
        """
        return PlannerSettings(self.json['planner_settings'], self._api)

    @property
    def finder_settings(self):
        """
        Retrieve finder settings for distro.

        :return: finder settings.
        """
        return FinderSettings(self.json['finder_settings'], self._api)



from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib


class ManifestModule(_BaseEvergreenObject):
    """
    Represents a module in the evergreen manifest.
    """

    branch = evg_attrib('branch')
    repo = evg_attrib('repo')
    revision = evg_attrib('revision')
    owner = evg_attrib('owner')
    url = evg_attrib('url')

    def __init__(self, name, json, api):
        """
        Create an instance of an evergreen manifest module.

        :param json: json representing manifest.
        :param api: evergreen api object
        """
        super(ManifestModule, self).__init__(json, api)
        self.name = name


class Manifest(_BaseEvergreenObject):
    """
    Representation of an evergreen manifest.
    """

    id = evg_attrib('id')
    revision = evg_attrib('revision')
    project = evg_attrib('project')
    branch = evg_attrib('branch')

    def __init__(self, json, api):
        """
        Create an instance of an evergreen version manifest.

        :param json: json representing manifest.
        :param api: evergreen api object.
        """
        super(Manifest, self).__init__(json, api)

    @property
    def modules(self):
        """Map of modules in this manifest."""
        if 'modules' not in self.json:
            return None

        modules = self.json['modules'].items()

        return {
            module_key: ManifestModule(module_key, module_value, self._api)
            for module_key, module_value in modules
        }



from __future__ import absolute_import

from enum import Enum
import json
import yaml

import click

from evergreen.api import EvergreenApi


DisplayFormat = Enum('DisplayFormat', 'human json yaml')


def fmt_output(fmt, data):
    """
    Convert the given data into the specified format.

    :param fmt: DisplayFormat to use.
    :param data: Data to convertn.
    :return: Data is specified format.
    """
    if fmt == DisplayFormat.json:
        return json.dumps(data, indent=4)
    if fmt == DisplayFormat.yaml:
        return yaml.safe_dump(data)
    return data


@click.group()
@click.option('--json', 'display_format', flag_value=DisplayFormat.json,
              help='Write output in json.')
@click.option('--yaml', 'display_format', flag_value=DisplayFormat.yaml,
              help='Write output in yaml.')
@click.option('--human-readable', 'display_format', flag_value=DisplayFormat.human, default=True,
              help='Write output in a human readable format.')
@click.pass_context
def cli(ctx, display_format):
    ctx.ensure_object(dict)
    ctx.obj['api'] = EvergreenApi.get_api(use_config_file=True)
    ctx.obj['format'] = display_format


@cli.command()
@click.pass_context
def list_hosts(ctx):
    api = ctx.obj['api']
    fmt = ctx.obj['format']
    host_list = api.all_hosts()
    click.echo(fmt_output(fmt, [host.json for host in host_list]))


@cli.command()
@click.pass_context
@click.option('-p', '--project', required=True)
@click.option('-l', '--limit', type=int)
def list_patches(ctx, project, limit):
    api = ctx.obj['api']
    fmt = ctx.obj['format']
    patches = []
    for i, p in enumerate(api.patches_by_project(project)):
        patches.append(p.json)
        if limit and i > limit:
            break
    click.echo(fmt_output(fmt, patches))


@cli.command()
@click.pass_context
def list_projects(ctx):
    api = ctx.obj['api']
    fmt = ctx.obj['format']
    project_list = api.all_projects()
    projects = [project.json for project in project_list]
    click.echo(fmt_output(fmt, projects))


@cli.command()
@click.pass_context
@click.option('-a', '--after-date', required=True)
@click.option('-b', '--before-date', required=True)
@click.option('-p', '--project', required=True)
@click.option('-d', '--distros', multiple=True)
@click.option('--group-by')
@click.option('-g', '--group-num-days')
@click.option('-r', '--requesters', multiple=True)
@click.option('-s', '--sort')
@click.option('--tests', multiple=True)
@click.option('-t', '--tasks', multiple=True)
@click.option('-v', '--variants', multiple=True)
def test_stats(ctx, after_date, before_date, project, distros, group_by, group_num_days, requesters,
               sort, tests, tasks, variants):
    api = ctx.obj['api']
    fmt = ctx.obj['format']

    test_stat_list = api.test_stats_by_project(project, after_date, before_date, group_num_days,
                                               requesters, tests, tasks, variants, distros,
                                               group_by, sort)
    test_statistics = [t.json for t in test_stat_list]
    click.echo(fmt_output(fmt, test_statistics))


@cli.command()
@click.pass_context
@click.option('-a', '--after-date', required=True)
@click.option('-b', '--before-date', required=True)
@click.option('-p', '--project', required=True)
@click.option('-d', '--distros', multiple=True)
@click.option('--group-by')
@click.option('-g', '--group-num-days')
@click.option('-r', '--requesters', multiple=True)
@click.option('-s', '--sort')
@click.option('-t', '--tasks', multiple=True)
@click.option('-v', '--variants', multiple=True)
def task_stats(ctx, after_date, before_date, project, distros, group_by, group_num_days, requesters,
               sort, tasks, variants):
    api = ctx.obj['api']
    fmt = ctx.obj['format']

    task_stat_list = api.task_stats_by_project(project, after_date, before_date, group_num_days,
                                               requesters, tasks, variants, distros,
                                               group_by, sort)
    task_statistics = [t.json for t in task_stat_list]
    click.echo(fmt_output(fmt, task_statistics))


@cli.command()
@click.pass_context
@click.option('-a', '--after-date')
@click.option('-b', '--before-date')
@click.option('-p', '--project', required=True)
@click.option('-d', '--distros', multiple=True)
@click.option('--group-by')
@click.option('-g', '--group-num-days', default=30)
@click.option('-r', '--requesters', multiple=True)
@click.option('-s', '--sort')
@click.option('-t', '--tasks', multiple=True, required=True)
@click.option('-v', '--variants', multiple=True)
def task_reliability(ctx, after_date, before_date, project, distros, group_by, group_num_days,
                     requesters, sort, tasks, variants):
    api = ctx.obj['api']
    fmt = ctx.obj['format']

    task_reliability_list = api.task_reliability_by_project(project, after_date, before_date,
                                                            group_num_days, requesters, tasks,
                                                            variants, distros, group_by, sort)
    task_reliability_scores = [t.json for t in task_reliability_list]
    click.echo(fmt_output(fmt, task_reliability_scores))


@cli.command()
@click.pass_context
@click.option('-v', '--version', 'version_id', required=True)
@click.option('--builds', is_flag=True, default=False, help='Include builds of version in output')
def version_stats(ctx, version_id, builds):
    """
    Collect stats for the given evergreen version.

    :param ctx: Command context.
    :param version_id: Id of version to analyze.
    :param builds: Include builds of version in output.
    """
    api = ctx.obj['api']
    fmt = ctx.obj['format']

    version = api.version_by_id(version_id)
    if fmt == DisplayFormat.human:
        click.echo(version.get_metrics())
    else:
        click.echo(fmt_output(fmt, version.get_metrics().as_dict(include_children=builds)))


@cli.command()
@click.pass_context
@click.option('-b', '--build', 'build_id', required=True)
@click.option('--tasks', is_flag=True, default=False, help='Include tasks of build in output')
def build_stats(ctx, build_id, tasks):
    """
    Collect stats for the given evergreen build.

    :param ctx: Command context.
    :param build_id: Id of build to analyze.
    :param tasks: If true include tasks in output.
    """
    api = ctx.obj['api']
    fmt = ctx.obj['format']

    build = api.build_by_id(build_id)
    if fmt == DisplayFormat.human:
        click.echo(build.get_metrics())
    else:
        click.echo(fmt_output(fmt, build.get_metrics().as_dict(include_children=tasks)))


def main():
    return cli(obj={})



# -*- encoding: utf-8 -*-
"""Stats representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_date_attrib


class TestStats(_BaseEvergreenObject):
    """Representation of an Evergreen test stats object."""

    test_file = evg_attrib('test_file')
    task_name = evg_attrib('task_name')
    test_name = evg_attrib('test_name')
    variant = evg_attrib('variant')
    distro = evg_attrib('distro')
    evg_date_attrib('date')
    num_pass = evg_attrib('num_pass')
    num_fail = evg_attrib('num_fail')
    avg_duration_pass = evg_attrib('avg_duration_pass')

    def __init__(self, json, api):
        """
        Create an instance of a test stats object.

        :param json: json version of object.
        """
        super(TestStats, self).__init__(json, api)


class TaskStats(_BaseEvergreenObject):
    """Representation of an Evergreen task stats object."""

    test_file = evg_attrib('test_file')
    task_name = evg_attrib('task_name')
    variant = evg_attrib('variant')
    distro = evg_attrib('distro')
    evg_date_attrib('date')
    num_pass = evg_attrib('num_pass')
    num_fail = evg_attrib('num_fail')
    avg_duration_pass = evg_attrib('avg_duration_pass')

    def __init__(self, json, api):
        """
        Create an instance of a test stats object.

        :param json: json version of object.
        """
        super(TaskStats, self).__init__(json, api)



# -*- encoding: utf-8 -*-
"""Test representation of evergreen."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib, evg_datetime_attrib


class Logs(_BaseEvergreenObject):
    """Representation of test logs from evergreen."""

    url = evg_attrib('url')
    line_num = evg_attrib('line_num')
    url_raw = evg_attrib('url_raw')
    log_id = evg_attrib('log_id')

    def __init__(self, json, api):
        """Create an instance of a Test log."""
        super(Logs, self).__init__(json, api)


class Tst(_BaseEvergreenObject):
    """
    Representation of a test object from evergreen.
    """

    task_id = evg_attrib('task_id')
    status = evg_attrib('status')
    test_file = evg_attrib('test_file')
    exit_code = evg_attrib('exit_code')
    start_time = evg_datetime_attrib('start_time')
    end_time = evg_datetime_attrib('end_time')

    def __init__(self, json, api):
        """Create an instance of a Test object."""
        super(Tst, self).__init__(json, api)

    @property
    def logs(self):
        """
        Get the log object for the given test.

        :return: log object for test.
        """
        return Logs(self.json['logs'], self._api)



# -*- encoding: utf-8 -*-
"""Evergreen representation of a project."""
from __future__ import absolute_import

from evergreen.base import _BaseEvergreenObject, evg_attrib


class Project(_BaseEvergreenObject):
    """Representation of an Evergreen project."""
    batch_time = evg_attrib('batch_time')
    branch_name = evg_attrib('branch_name')
    display_name = evg_attrib('display_name')
    enabled = evg_attrib('enabled')
    identifier = evg_attrib('identifier')
    owner_name = evg_attrib('owner_name')
    private = evg_attrib('private')
    remote_path = evg_attrib('remote_path')
    repo_name = evg_attrib('repo_name')
    tracked = evg_attrib('tracked')
    deactivated_previous = evg_attrib('deactivate_previous')
    admins = evg_attrib('admins')
    tracks_push_events = evg_attrib('tracks_push_events')
    pr_testing_enabled = evg_attrib('pr_testing_enabled')
    commit_queue = evg_attrib('commit_queue')

    def __init__(self, json, api):
        """
        Create an instance of an evergreen project.

        :param json: json representing project.
        :param api: evergreen api object.
        """
        super(Project, self).__init__(json, api)

    def __str__(self):
        """

        :return:
        """
        return self.identifier



"""Useful utilities for interacting with Evergreen."""
from datetime import datetime

EVG_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
EVG_DATE_FORMAT = '%Y-%m-%d'
EVG_DATE_INPUT_FORMAT = '"%Y-%m-%dT%H:%M:%S.000Z"'


def parse_evergreen_datetime(evg_date):
    """
    Convert an evergreen datetime string into a datetime object.

    :param evg_date: String to convert to a datetime.
    :return datetime version of date.
    """
    if not evg_date:
        return None
    return datetime.strptime(evg_date, EVG_DATETIME_FORMAT)


def format_evergreen_datetime(when):
    """
    Convert a datetime object into an evergreen consumable string.

    :param when: datetime to convert.
    :return: string evergreen can understand.
    """
    return when.strftime(EVG_DATE_INPUT_FORMAT)


def evergreen_input_to_output(input_date):
    """
    Convert a date from evergreen to a date to send back to evergreen.

    :param input_date: date to convert.
    :return: date to send to evergreen.
    """
    return format_evergreen_datetime(parse_evergreen_datetime(input_date))


def parse_evergreen_date(evg_date):
    """
    Convert an evergreen date string into a date object.

    :param evg_date: String to convert to a date.
    :return: date version of date.
    """
    if not evg_date:
        return None
    return datetime.strptime(evg_date, EVG_DATE_FORMAT).date()



# -*- encoding: utf-8 -*-
"""Metrics for Evergreen builds."""
from __future__ import absolute_import
from __future__ import division

from structlog import get_logger

from evergreen.errors.exceptions import ActiveTaskMetricsException

LOGGER = get_logger(__name__)


class BuildMetrics(object):
    """Metrics about an evergreen build."""

    def __init__(self, build):
        """
        Create an instance of build metrics.

        :param build: Build to analyze.
        """
        self.build = build

        self.success_count = 0
        self.failure_count = 0
        self.undispatched_count = 0
        self.timed_out_count = 0
        self.system_failure_count = 0

        self.estimated_build_costs = 0
        self.total_processing_time = 0

        self._create_times = []
        self._start_times = []
        self._finish_times = []

        self.task_list = None

    def calculate(self, task_filter_fn=None):
        """
        Calculate metrics for the given build.

        :param task_filter_fn: function to filter tasks included for metrics, should accept a task
                               argument.
        :returns: self.
        """
        all_tasks = self.build.get_tasks()
        filtered_task_list = all_tasks
        if task_filter_fn:
            filtered_task_list = [task for task in filtered_task_list if task_filter_fn(task)]

        self.task_list = filtered_task_list

        # We want to track display tasks, but not use them for metrics since they are just
        # containers to other tasks.
        filtered_task_list = [task for task in self.task_list if not task.display_only]
        for task in filtered_task_list:
            self._count_task(task)

        return self

    @property
    def total_tasks(self):
        """
        Get the total tasks in the build.
        :return: total tasks.
        """
        return self.success_count + self.failure_count + self.undispatched_count

    @property
    def pct_tasks_success(self):
        """
        Get the percentage of successful tasks.

        :return: Percentage of successful tasks.
        """
        return self._percent_tasks(self.success_count)

    @property
    def pct_tasks_undispatched(self):
        """
        Get the percentage of undispatched tasks.

        :return: Percentage of undispatched tasks.
        """
        return self._percent_tasks(self.undispatched_count)

    @property
    def pct_tasks_failed(self):
        """
        Get the percentage of failed tasks.

        :return: Percentage of failed tasks.
        """
        return self._percent_tasks(self.failure_count)

    @property
    def pct_tasks_timed_out(self):
        """
        Get the percentage of timeout tasks.

        :return: Percentage of timeout tasks.
        """
        return self._percent_tasks(self.timed_out_count)

    @property
    def pct_tasks_system_failure(self):
        """
        Get the percentage of system failure tasks.

        :return: Percentage of system failure tasks.
        """
        return self._percent_tasks(self.system_failure_count)

    @property
    def create_time(self):
        """
        Time the first task of the build was created.

        :return: Time first task was created.
        """
        if self._create_times:
            return min(self._create_times)
        return None

    @property
    def start_time(self):
        """
        Time first task of build was started.

        :return: Time first task was started.
        """
        if self._start_times:
            return min(self._start_times)
        return None

    @property
    def end_time(self):
        """
        Time last task of build was completed.

        :return: Time last task was completed.
        """
        if self._finish_times:
            return max(self._finish_times)
        return None

    @property
    def makespan(self):
        """
        Wall clock duration of build.

        :return: Timedelta duration of build.
        """
        if self.start_time and self.end_time:
            return self.end_time - self.start_time
        return None

    @property
    def wait_time(self):
        """
        Wall clock duration until build was started.

        :return: Timedelta duration until build was started.
        """
        if self.start_time and self.create_time:
            return self.start_time - self.create_time
        return None

    def _percent_tasks(self, n_tasks):
        """
        Calculate the percent of n_tasks out of total.

        :param n_tasks: Number of tasks to calculate percent of.
        :return: percentage n_tasks is out of total tasks.
        """
        if self.total_tasks == 0:
            return 0
        return n_tasks / self.total_tasks

    def _count_task(self, task):
        """
        Add stats for the given task to the metrics.
        :param task: Task to add.
        """
        if task.is_undispatched():
            self.undispatched_count += 1
            return  # An 'undispatched' task has no useful stats.

        if task.is_active():
            LOGGER.warning('Active task found during metrics collection', task_id=task.task_id)
            raise ActiveTaskMetricsException(task, 'Task in progress during metrics collection')

        if task.is_success():
            self.success_count += 1
        else:
            self.failure_count += 1
            if task.is_system_failure():
                self.system_failure_count += 1
            if task.is_timeout():
                self.timed_out_count += 1

        if task.ingest_time:
            self._create_times.append(task.ingest_time)
        else:
            self._create_times.append(task.start_time)

        if task.start_time:
            self._finish_times.append(task.finish_time)

        if task.start_time:
            self._start_times.append(task.start_time)

        self.estimated_build_costs += task.estimated_cost
        self.total_processing_time += task.time_taken_ms / 1000

    def as_dict(self, include_children=False):
        """
        Provide a dictionary representation.

        :param include_children: Include child tasks in dictionary.
        :return: Dictionary of metrics.
        """
        metric = {
            'build': self.build.id,
            'total_processing_time': self.total_processing_time,
            'makespan': self.makespan.total_seconds(),
            'wait_time': self.wait_time.total_seconds(),
            'total_tasks': self.total_tasks,
            'success_count': self.success_count,
            'pct_tasks_success': self.pct_tasks_success,
            'undispatched_count': self.undispatched_count,
            'pct_tasks_undispatched': self.pct_tasks_undispatched,
            'failure_count': self.failure_count,
            'pct_tasks_failed': self.pct_tasks_failed,
            'timed_out_count': self.timed_out_count,
            'system_failure_count': self.system_failure_count,
            'estimated_build_costs': self.estimated_build_costs,
        }

        if include_children:
            metric['tasks'] = [task.json for task in self.task_list]

        return metric

    def __str__(self):
        """
        Create string version of metrics.

        :return: String version of the metrics.
        """
        return """Build Id: {build_id}
        Total Processing Time: {total_processing_time:.2f}s ({total_processing_time_min:.2f}m)
        Makespan: {makespan:.2f}s ({makespan_min:.2f}m)
        Wait Time: {waittime:.2f}s ({waittime_min:.2f}m)
        Total Tasks: {total_tasks}
        Successful Tasks: {success_count} ({success_pct:.2%})
        Undispatched Tasks: {undispatched_count} ({undispatched_pct:.2%})
        Failed Tasks: {failed_count} ({failed_pct:.2%})
        Timeout Tasks: {timeout_count} ({timeout_pct:.2%})
        System Failure Tasks: {system_failure_count} ({system_failure_pct:.2%})
        Estimated Build Costs: {est_build_costs:.3f}
        """.format(
            build_id=self.build.id,
            total_processing_time=self.total_processing_time,
            total_processing_time_min=(self.total_processing_time / 60),
            makespan=(self.makespan.total_seconds()),
            makespan_min=(self.makespan.total_seconds() / 60),
            waittime=(self.wait_time.total_seconds()),
            waittime_min=(self.wait_time.total_seconds() / 60),
            total_tasks=self.total_tasks,
            success_count=self.success_count,
            success_pct=self.pct_tasks_success,
            undispatched_count=self.undispatched_count,
            undispatched_pct=self.pct_tasks_undispatched,
            failed_count=self.failure_count,
            failed_pct=self.pct_tasks_failed,
            timeout_count=self.timed_out_count,
            timeout_pct=self.pct_tasks_timed_out,
            system_failure_count=self.system_failure_count,
            system_failure_pct=self.pct_tasks_system_failure,
            est_build_costs=self.estimated_build_costs
        ).rstrip()
